@@ -132,12 +132,20 @@ add_account ()
     chown -R ${GUEST_USER}:${GUEST_USER} ${GUEST_HOME}
   fi
 
-  # delay the launch of info dialog
-  echo "export DIALOG_SLEEP=4" >> ${GUEST_HOME}/.profile
-
   # setup session
   su ${GUEST_USER} -c "env HOME=${GUEST_HOME} site_gs=${site_gs} ${dist_gs}/setup.sh"
 
+  # set possible local guest session preferences
+  source_local_prefs() {
+    local USER=${GUEST_USER}
+    local HOME=${GUEST_HOME}
+    . ${site_gs}/prefs.sh
+    chown -R ${USER}:${USER} ${HOME}
+  }
+  if [ -f ${site_gs}/prefs.sh ]; then
+    source_local_prefs
+  fi
+
   echo ${GUEST_USER}
 }
 
