@@ -196,9 +196,19 @@ set_seat_properties (Seat *seat, const gchar *seat_name)
 static void
 signal_cb (Process *process, int signum)
 {
-    g_debug ("Caught %s signal, shutting down", g_strsignal (signum));
-    display_manager_stop (display_manager);
-    // FIXME: Stop XDMCP server
+    switch (signum)
+    {
+    case SIGINT:
+    case SIGTERM:
+        g_debug ("Caught %s signal, shutting down", g_strsignal (signum));
+        display_manager_stop (display_manager);
+        // FIXME: Stop XDMCP server
+        break;
+    case SIGUSR1:
+    case SIGUSR2:
+    case SIGHUP:
+        break;
+    }
 }
 
 static void
@@ -208,6 +218,17 @@ display_manager_stopped_cb (DisplayManager *display_manager)
     g_main_loop_quit (loop);
 }
 
+static Seat *
+create_seat (const gchar *module_name, const gchar *name)
+{
+    if (strcmp (module_name, "xlocal") == 0) {
+        g_warning ("Seat type 'xlocal' is deprecated, use 'type=local' instead");
+        return seat_new ("local", name);
+    }
+    else
+        return seat_new (module_name, name);
+}
+
 static void
 display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 {
@@ -227,7 +248,7 @@ display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 
         if (!next_seat)
         {
-            next_seat = seat_new (*iter, seat_get_name (seat));
+            next_seat = create_seat (*iter, seat_get_name (seat));
             g_string_assign (next_types, *iter);
         }
         else
@@ -344,7 +365,7 @@ handle_display_manager_call (GDBusConnection       *connection,
 
         g_debug ("Adding local X seat :%d", display_number);
 
-        seat = seat_new ("xremote", "xremote0"); // FIXME: What to use for a name?
+        seat = create_seat ("xremote", "xremote0"); // FIXME: What to use for a name?
         if (seat)
         {
             gchar *display_number_string;
@@ -879,7 +900,7 @@ bus_acquired_cb (GDBusConnection *connection,
     /* Start the XDMCP server */
     if (config_get_boolean (config_get_instance (), "XDMCPServer", "enabled"))
     {
-        gchar *key_name, *key = NULL, *listen_address;
+        gchar *key_name, *key = NULL, *listen_address, *hostname;
 
         xdmcp_server = xdmcp_server_new ();
         if (config_has_key (config_get_instance (), "XDMCPServer", "port"))
@@ -892,6 +913,9 @@ bus_acquired_cb (GDBusConnection *connection,
         listen_address = config_get_string (config_get_instance (), "XDMCPServer", "listen-address");
         xdmcp_server_set_listen_address (xdmcp_server, listen_address);
         g_free (listen_address);
+        hostname = config_get_string (config_get_instance (), "XDMCPServer", "hostname");
+        xdmcp_server_set_hostname (xdmcp_server, hostname);
+        g_free (hostname);
         g_signal_connect (xdmcp_server, XDMCP_SERVER_SIGNAL_NEW_SESSION, G_CALLBACK (xdmcp_session_cb), NULL);
 
         key_name = config_get_string (config_get_instance (), "XDMCPServer", "key");
@@ -1007,7 +1031,7 @@ add_login1_seat (Login1Seat *login1_seat)
     g_list_free_full (config_sections, g_free);
 
     for (type = types; !seat && type && *type; type++)
-        seat = seat_new (*type, seat_name);
+        seat = create_seat (*type, seat_name);
     g_strfreev (types);
 
     if (seat)
@@ -1025,15 +1049,14 @@ add_login1_seat (Login1Seat *login1_seat)
             seat_set_property (seat, "exit-on-failure", "true");
     }
     else
-        g_debug ("Unable to create seat: %s", seat_name);
-
-    if (seat)
     {
-        started = display_manager_add_seat (display_manager, seat);
-        if (!started)
-            g_debug ("Failed to start seat: %s", seat_name);
+        g_debug ("Unable to create seat: %s", seat_name);
+        return FALSE;
     }
 
+    started = display_manager_add_seat (display_manager, seat);
+    if (!started)
+        g_debug ("Failed to start seat: %s", seat_name);
     g_object_unref (seat);
 
     return started;
@@ -1100,9 +1123,10 @@ login1_active_session_changed_cb (Login1Seat *login1_seat, const gchar *login1_s
     if (seat)
     {
         Session *active_session;
-        active_session = seat_get_expected_active_session (seat);
 
-        if (g_strcmp0 (login1_session_id, session_get_login1_session_id (active_session)) == 0)
+        active_session = seat_get_expected_active_session (seat);
+        if (active_session != NULL &&
+            g_strcmp0 (login1_session_id, session_get_login1_session_id (active_session)) == 0)
         {
             // Session is already active
             g_debug ("Session %s is already active", login1_session_id);
@@ -1385,7 +1409,7 @@ main (int argc, char **argv)
     if (!config_has_key (config_get_instance (), "LightDM", "backup-logs"))
         config_set_boolean (config_get_instance (), "LightDM", "backup-logs", TRUE);
     if (!config_has_key (config_get_instance (), "Seat:*", "type"))
-        config_set_string (config_get_instance (), "Seat:*", "type", "xlocal");
+        config_set_string (config_get_instance (), "Seat:*", "type", "local");
     if (!config_has_key (config_get_instance (), "Seat:*", "pam-service"))
         config_set_string (config_get_instance (), "Seat:*", "pam-service", "lightdm");
     if (!config_has_key (config_get_instance (), "Seat:*", "pam-autologin-service"))
@@ -1411,9 +1435,9 @@ main (int argc, char **argv)
     if (!config_has_key (config_get_instance (), "Seat:*", "greeter-show-remote-login"))
         config_set_boolean (config_get_instance (), "Seat:*", "greeter-show-remote-login", TRUE);
     if (!config_has_key (config_get_instance (), "Seat:*", "greeter-session"))
-        config_set_string (config_get_instance (), "Seat:*", "greeter-session", GREETER_SESSION);
+        config_set_string (config_get_instance (), "Seat:*", "greeter-session", DEFAULT_GREETER_SESSION);
     if (!config_has_key (config_get_instance (), "Seat:*", "user-session"))
-        config_set_string (config_get_instance (), "Seat:*", "user-session", USER_SESSION);
+        config_set_string (config_get_instance (), "Seat:*", "user-session", DEFAULT_USER_SESSION);
     if (!config_has_key (config_get_instance (), "Seat:*", "session-wrapper"))
         config_set_string (config_get_instance (), "Seat:*", "session-wrapper", "lightdm-session");
     if (!config_has_key (config_get_instance (), "LightDM", "log-directory"))
@@ -1431,6 +1455,8 @@ main (int argc, char **argv)
         config_set_string (config_get_instance (), "LightDM", "remote-sessions-directory", REMOTE_SESSIONS_DIR);
     if (!config_has_key (config_get_instance (), "LightDM", "greeters-directory"))
         config_set_string (config_get_instance (), "LightDM", "greeters-directory", GREETERS_DIR);
+    if (!config_has_key (config_get_instance (), "XDMCPServer", "hostname"))
+        config_set_string (config_get_instance (), "XDMCPServer", "hostname", g_get_host_name ());
 
     /* Override defaults */
     if (log_dir)
@@ -1517,7 +1543,7 @@ main (int argc, char **argv)
             types = config_get_string_list (config_get_instance (), "Seat:*", "type");
             for (type = types; type && *type; type++)
             {
-                seat = seat_new (*type, "seat0");
+                seat = create_seat (*type, "seat0");
                 if (seat)
                     break;
             }
